/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gwf_projdata_signal.h"
#include <gtk/gtk.h>
#include <gwf_lib.h>

enum {
   OFRSIG_ADD,
   OFRSIG_CHANGE,
   OFRSIG_REMOVE,

   OFRSIG_PARAM_ADD,
   OFRSIG_PARAM_CHANGE,
   OFRSIG_PARAM_REMOVE,

   LAST_SIGNAL
};

typedef struct _GwfOfrsigForeach	GwfOfrsigForeach;
typedef struct _GwfSigparamForeach	GwfSigparamForeach;
typedef struct _GwfSignalData		GwfSignalData;

struct _GwfOfrsigForeach
{
  GwfOfrsigForeachFunc	func;
  gpointer		user_data;
};

struct _GwfSigparamForeach
{
  GwfSigparamForeachFunc	func;
  gpointer			user_data;
};

struct _GwfSignalData
{
  GList		*list;
};

typedef void (*GtkSignal_NONE__STRING_STRING) (GtkObject *, gchar *,gchar *, gpointer);

static void		gwf_project_signal_struct_free	(gpointer data);
static gint		gwf_offersig_cmp		(gconstpointer a,gconstpointer b);
static gint		gwf_param_cmp 			(gconstpointer a,gconstpointer b);
static void		foreach_ofrsig 			(gpointer data,gpointer user_data);
static void		foreach_sigparam 		(gpointer data,gpointer user_data);
static void		gwf_param_list_free 		(GList **list);
static void		gwf_offersig_list_free 		(GList **list);
static offersig*	gwf_project_get_ofrsig 		(GwfProject *project,gchar *name);
static GList*		gwf_project_get_ofrsig_list 	(GwfProject *project,gchar *name);
static sigparam*	gwf_ofrsig_get_param 		(offersig *signal,gchar *name);
static GList*		gwf_ofrsig_get_param_list	(offersig *signal,gchar *name);
static GwfSignalData*	gwf_project_signal_get_struct	(GwfProject *project);

static void		gwf_marshal_NONE_STRING_STRING	(GtkObject		*object,
   							 GtkSignalFunc		func,
                                                         gpointer		func_data,
                                                         GtkArg			*args);


static guint project_signals[LAST_SIGNAL] = { 0 };

GwfProjData*
gwf_project_signal_new (GwfProject *project)
{
  GwfProjData		*tmp;
  GwfSignalData		*data;
  static gint		add_signals = 1;

  g_return_val_if_fail(project != NULL,NULL);

  tmp = gwf_proj_data_new("Signals");

  data = g_malloc(sizeof(GwfSignalData));
  data->list = NULL;
  gwf_proj_data_set_data(tmp,(gpointer)data);
  gwf_proj_data_set_free_func(tmp,gwf_project_signal_struct_free);

  if (add_signals)
  {
    project_signals[OFRSIG_ADD] = gwf_project_new_signal(project,"offer_signal_added",gtk_marshal_NONE__STRING,GTK_TYPE_NONE,1,GTK_TYPE_STRING);
    if (!project_signals[OFRSIG_ADD])
    {
      gwf_proj_data_free(tmp);
      return NULL;
    }

    project_signals[OFRSIG_CHANGE] = gwf_project_new_signal(project,"offer_signal_changed",gtk_marshal_NONE__STRING,GTK_TYPE_NONE,1,GTK_TYPE_STRING);
    if (!project_signals[OFRSIG_CHANGE])
    {
      gwf_proj_data_free(tmp);
      return NULL;
    }

    project_signals[OFRSIG_REMOVE] = gwf_project_new_signal(project,"offer_signal_removed",gtk_marshal_NONE__STRING,GTK_TYPE_NONE,1,GTK_TYPE_STRING);
    if (!project_signals[OFRSIG_REMOVE])
    {
      gwf_proj_data_free(tmp);
      return NULL;
    }

    project_signals[OFRSIG_PARAM_ADD] = gwf_project_new_signal(project,"ofrsig_param_added",gwf_marshal_NONE_STRING_STRING,GTK_TYPE_NONE,2,GTK_TYPE_STRING,GTK_TYPE_STRING);
    if (!project_signals[OFRSIG_PARAM_ADD])
    {
      gwf_proj_data_free(tmp);
      return NULL;
    }

    project_signals[OFRSIG_PARAM_CHANGE] = gwf_project_new_signal(project,"ofrsig_param_changed",gwf_marshal_NONE_STRING_STRING,GTK_TYPE_NONE,2,GTK_TYPE_STRING,GTK_TYPE_STRING);
    if (!project_signals[OFRSIG_PARAM_CHANGE])
    {
      gwf_proj_data_free(tmp);
      return NULL;
    }

    project_signals[OFRSIG_PARAM_REMOVE] = gwf_project_new_signal(project,"ofrsig_param_removed",gwf_marshal_NONE_STRING_STRING,GTK_TYPE_NONE,2,GTK_TYPE_STRING,GTK_TYPE_STRING);
    if (!project_signals[OFRSIG_PARAM_CHANGE])
    {
      gwf_proj_data_free(tmp);
      return NULL;
    }

    add_signals = 0;
  }

  return tmp;
}

static void
gwf_project_signal_struct_free (gpointer data)
{
  gwf_offersig_list_free(&((GwfSignalData*)data)->list);
  g_free(data);
}

/**************************/
/* Offer signal functions */
/**************************/

void
gwf_project_add_ofrsig (GwfProject *project,offersig *signal)
{
  offersig	*tmp_sig;
  sigparam	param;
  GList		*tmp_list;
  GwfSignalData	*data;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(signal != NULL);

  data = gwf_project_signal_get_struct(project);
  g_return_if_fail(data != NULL);
  
  if (gwf_project_get_ofrsig_list(project,signal->name))
    return;

  tmp_sig = g_malloc(sizeof(offersig));
  tmp_sig->name = g_strdup(signal->name);
  tmp_sig->retval = g_strdup(signal->retval);
  tmp_sig->rvtype = signal->rvtype;
  tmp_sig->param = NULL;

  data->list = g_list_append(data->list,(gpointer)tmp_sig);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_ADD],tmp_sig->name);

  for (tmp_list = signal->param ; tmp_list ; tmp_list = g_list_next(tmp_list))
  {
    param.name = ((sigparam*)tmp_list->data)->name;
    param.type = ((sigparam*)tmp_list->data)->type;
    param.gtktype = ((sigparam*)tmp_list->data)->gtktype;

    gwf_project_ofrsig_add_param (project,signal->name,&param);
  }
  
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_new_ofrsig (GwfProject *project,gchar *name)
{
  offersig	tmp_sig;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(name != NULL);
  
  tmp_sig.name = name;
  tmp_sig.retval = NULL;
  tmp_sig.rvtype = GTK_TYPE_INVALID;
  tmp_sig.param = NULL;
  
  gwf_project_add_ofrsig(project,&tmp_sig);
}

void		gwf_project_del_ofrsig		(GwfProject *project,gchar *name)
{
  GList		*tmp_list;
  GwfSignalData	*data;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(name != NULL);
  
  data = gwf_project_signal_get_struct(project);
  g_return_if_fail(data != NULL);

  tmp_list = gwf_project_get_ofrsig_list(project,name);
  if (!tmp_list)
    return;
    
  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_REMOVE],name);

  g_free(((offersig*)tmp_list->data)->name);
  g_free(((offersig*)tmp_list->data)->retval);
  gwf_param_list_free( &((offersig*)tmp_list->data)->param );
  g_free(tmp_list->data);
  
  data->list = g_list_remove(data->list,tmp_list->data);

  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_set_name (GwfProject *project,gchar *name,gchar *newname)
{
  offersig		*tmp_sig;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(name != NULL);
  g_return_if_fail(newname != NULL);
  
  if (gwf_project_get_ofrsig_list(project,newname))
    return;
    
  tmp_sig = gwf_project_get_ofrsig(project,name);
  if (!tmp_sig)
    return;
    
  g_free(tmp_sig->name);
  tmp_sig->name = g_strdup(newname);

  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_CHANGE],tmp_sig->name);
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_set_retval (GwfProject *project,gchar *name,gchar *retval)
{
  offersig		*tmp_sig;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(name != NULL);

  tmp_sig = gwf_project_get_ofrsig(project,name);
  if (!tmp_sig)
    return;
    
  g_free(tmp_sig->retval);
  tmp_sig->retval = g_strdup(retval);

  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_CHANGE],tmp_sig->name);
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_set_rvtype (GwfProject *project,gchar *name,GtkType type)
{
  offersig		*tmp_sig;

  g_return_if_fail(project != NULL);
  g_return_if_fail(name != NULL);

  tmp_sig = gwf_project_get_ofrsig(project,name);
  if (!tmp_sig)
    return;

  tmp_sig->rvtype = type;

  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_CHANGE],tmp_sig->name);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_ofrsig_get_retval (GwfProject *project,gchar *name)
{
  offersig		*tmp_sig;
  
  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  tmp_sig = gwf_project_get_ofrsig(project,name);
  return (tmp_sig) ? tmp_sig->retval : NULL;
}

GtkType
gwf_project_ofrsig_get_rvtype (GwfProject *project,gchar *name)
{
  offersig		*tmp_sig;

  g_return_val_if_fail(project != NULL,GTK_TYPE_INVALID);
  g_return_val_if_fail(name != NULL,GTK_TYPE_INVALID);

  tmp_sig = gwf_project_get_ofrsig(project,name);
  return (tmp_sig) ? tmp_sig->rvtype : GTK_TYPE_INVALID;
}

GList*
gwf_project_ofrsig_get_param (GwfProject *project,gchar *name)
{
  offersig		*tmp_sig;
  
  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  tmp_sig = gwf_project_get_ofrsig(project,name);
  return (tmp_sig) ? tmp_sig->param : NULL;
}

void
gwf_project_ofrsig_foreach (GwfProject *project,GwfOfrsigForeachFunc func,gpointer user_data)
{
  GwfOfrsigForeach	tmp;
  GwfSignalData		*data;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(func != NULL);

  data = gwf_project_signal_get_struct(project);
  g_return_if_fail(data != NULL);

  tmp.func = func;
  tmp.user_data = user_data;

  g_list_foreach(data->list,foreach_ofrsig,(gpointer)&tmp);
}

offersig*
gwf_project_ofrsig_find	(GwfProject *project,gchar *name)
{
  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  return gwf_project_get_ofrsig(project,name);
}

gint
gwf_project_ofrsig_count (GwfProject *project)
{
  GwfSignalData	*data;
  GList		*tmp;
  gint		n = 0;

  g_return_val_if_fail(project != NULL,0);

  data = gwf_project_signal_get_struct(project);
  g_return_val_if_fail(data != NULL,0);

  for ( tmp = data->list ; tmp ; tmp = tmp->next)
    n++;

  return n;
}

/************************************/
/* Offer signal parameter functions */
/************************************/

gint
gwf_ofrsig_add_param (offersig *signal,sigparam *param)
{
  sigparam		*tmp_param;
  
  g_return_val_if_fail(signal != NULL,0);
  g_return_val_if_fail(param != NULL,0);
  
  if (gwf_ofrsig_get_param_list(signal,param->name))
    return 0;
    
  tmp_param = g_malloc(sizeof(sigparam));
  tmp_param->name = g_strdup(param->name);
  tmp_param->type = g_strdup(param->type);
  tmp_param->gtktype = param->gtktype;
  
  signal->param = g_list_append(signal->param, (gpointer)tmp_param);
  
  return 1;
}

gint
gwf_ofrsig_new_param (offersig *signal,gchar *name)
{
  sigparam		param;

  g_return_val_if_fail(signal != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  
  param.name = name;
  param.type = NULL;
  param.gtktype = GTK_TYPE_INVALID;
  
  return gwf_ofrsig_add_param(signal,&param);
}

gint
gwf_ofrsig_del_param (offersig *signal,gchar *name)
{
  GList			*tmp_list;
  
  g_return_val_if_fail(signal != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  
  tmp_list = gwf_ofrsig_get_param_list(signal,name);
  if (!tmp_list)
    return 0;

  g_free(((sigparam*)tmp_list->data)->name);
  g_free(((sigparam*)tmp_list->data)->type);
  g_free(tmp_list->data);
  
  signal->param = g_list_remove(signal->param,tmp_list->data);
  
  return 1;
}

gint
gwf_ofrsig_param_set_name (offersig *signal,gchar *name,gchar *newname)
{
  sigparam		*tmp_param;

  g_return_val_if_fail(signal != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  g_return_val_if_fail(newname != NULL,0);
  

  if (gwf_ofrsig_get_param_list(signal,newname))
    return 0;
    
  tmp_param = gwf_ofrsig_get_param(signal,name);
  if (!tmp_param)
    return 0;
    
  g_free(tmp_param->name);
  tmp_param->name = g_strdup(newname);
  
  return 1;
}

gint
gwf_ofrsig_param_set_type (offersig *signal,gchar *name,gchar *type)
{
  sigparam		*tmp_param;
  
  g_return_val_if_fail(signal != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  
  tmp_param = gwf_ofrsig_get_param(signal,name);
  if (!tmp_param)
    return 0;
    
  g_free(tmp_param->type);
  tmp_param->type = g_strdup(type);

  return 1;
}

gint
gwf_ofrsig_param_set_gtktype (offersig *signal,gchar *name,GtkType type)
{
  sigparam		*tmp_param;

  g_return_val_if_fail(signal != NULL,0);
  g_return_val_if_fail(name != NULL,0);

  tmp_param = gwf_ofrsig_get_param(signal,name);
  if (!tmp_param)
    return 0;

  tmp_param->gtktype = type;

  return 1;
}

gchar*
gwf_ofrsig_param_get_type (offersig *signal,gchar *name)
{
  sigparam		*tmp_param;
  
  g_return_val_if_fail(signal != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  tmp_param = gwf_ofrsig_get_param(signal,name);
  return (tmp_param) ? tmp_param->type : NULL;
}

GtkType
gwf_ofrsig_param_get_gtktype (offersig *signal,gchar *name)
{
  sigparam		*tmp_param;

  g_return_val_if_fail(signal != NULL,GTK_TYPE_INVALID);
  g_return_val_if_fail(name != NULL,GTK_TYPE_INVALID);

  tmp_param = gwf_ofrsig_get_param(signal,name);
  return (tmp_param) ? tmp_param->gtktype : GTK_TYPE_INVALID;
}

void
gwf_ofrsig_param_foreach (offersig *signal,GwfSigparamForeachFunc func,gpointer user_data)
{
  GwfSigparamForeach	tmp;
  
  g_return_if_fail(signal != NULL);
  g_return_if_fail(func != NULL);

  tmp.func = func;
  tmp.user_data = user_data;

  g_list_foreach(signal->param,foreach_sigparam,(gpointer)&tmp);
}

sigparam*
gwf_ofrsig_param_find (offersig *signal,gchar *name)
{
  g_return_val_if_fail(signal != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  return gwf_ofrsig_get_param(signal,name);
}

void
gwf_ofrsig_free (offersig *signal)
{
  g_return_if_fail(signal != NULL);
  
  g_free(signal->name);
  g_free(signal->retval);
  gwf_param_list_free(&signal->param);
  g_free(signal);
}

/*************************************************/
/* Offer signal parameter functions with project */
/*************************************************/

void
gwf_project_ofrsig_add_param (GwfProject *project,gchar *signame,sigparam *param)
{
  offersig		*signal;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(signame != NULL);
  g_return_if_fail(param != NULL);
  
  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return;
  
  if (!gwf_ofrsig_add_param(signal,param))
    return;

  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_PARAM_ADD],signame,param->name);
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_new_param (GwfProject *project,gchar *signame,gchar *name)
{
  sigparam		param;
  
  param.name = name;
  param.type = NULL;
  
  gwf_project_ofrsig_add_param(project,signame,&param);
}

void		gwf_project_ofrsig_del_param		(GwfProject *project,gchar *signame,gchar *name)
{
  offersig		*signal;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(signame != NULL);
  g_return_if_fail(name != NULL);
  
  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return;
    
  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_PARAM_REMOVE],signame,name);

  if (!gwf_ofrsig_del_param(signal,name))
    return;
    
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_param_set_name (GwfProject *project,gchar *signame,gchar *name,gchar *newname)
{
  offersig		*signal;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(signame != NULL);
  g_return_if_fail(name != NULL);
  
  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return;
    
  if (!gwf_ofrsig_param_set_name(signal,name,newname))
    return;   

  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_PARAM_CHANGE],signame,newname);
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_param_set_type (GwfProject *project,gchar *signame,gchar *name,gchar *type)
{
  offersig		*signal;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(signame != NULL);
  g_return_if_fail(name != NULL);
  
  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return;

  if (!gwf_ofrsig_param_set_type(signal,name,type))
    return;
    
  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_PARAM_CHANGE],signame,name);
  gwf_project_set_changed(project,TRUE);
}

void
gwf_project_ofrsig_param_set_gtktype (GwfProject *project,gchar *signame,gchar *name,GtkType type)
{
  offersig		*signal;

  g_return_if_fail(project != NULL);
  g_return_if_fail(signame != NULL);
  g_return_if_fail(name != NULL);

  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return;

  if (!gwf_ofrsig_param_set_gtktype(signal,name,type))
    return;

  gtk_signal_emit(GTK_OBJECT(project),project_signals[OFRSIG_PARAM_CHANGE],signame,name);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_ofrsig_param_get_type (GwfProject *project,gchar *signame,gchar *name)
{
  offersig		*signal;
  
  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(signame != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return NULL;

  return gwf_ofrsig_param_get_type(signal,name);
}

GtkType
gwf_project_ofrsig_param_get_gtktype (GwfProject *project,gchar *signame,gchar *name)
{
  offersig		*signal;

  g_return_val_if_fail(project != NULL,GTK_TYPE_INVALID);
  g_return_val_if_fail(signame != NULL,GTK_TYPE_INVALID);
  g_return_val_if_fail(name != NULL,GTK_TYPE_INVALID);

  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return GTK_TYPE_INVALID;

  return gwf_ofrsig_param_get_gtktype(signal,name);
}

void
gwf_project_ofrsig_param_foreach (GwfProject *project,gchar *signame,GwfSigparamForeachFunc func,gpointer user_data)
{
  offersig		*signal;
  
  g_return_if_fail(project != NULL);
  g_return_if_fail(signame != NULL);
  g_return_if_fail(func != NULL);

  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return;
  
  gwf_ofrsig_param_foreach (signal,func,user_data);
}

sigparam*
gwf_project_ofrsig_param_find (GwfProject *project,gchar *signame,gchar *name)
{
  offersig		*signal;

  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(signame != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  signal = gwf_project_get_ofrsig(project,signame);
  if (!signal)
    return NULL;

  return gwf_ofrsig_get_param(signal,name);
}

/*****************************/
/* Internal search functions */
/*****************************/

static sigparam*
gwf_ofrsig_get_param (offersig *signal,gchar *name)
{
  GList		*tmp_list;
  
  g_return_val_if_fail(signal != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  tmp_list = gwf_ofrsig_get_param_list(signal,name);
  return (tmp_list) ? (sigparam*)(tmp_list->data) : NULL;
}

static GList*
gwf_ofrsig_get_param_list(offersig *signal,gchar *name)
{
  return g_list_find_custom(signal->param, (gpointer)name, (GCompareFunc)gwf_param_cmp);
}

static offersig*
gwf_project_get_ofrsig (GwfProject *project,gchar *name)
{
  GList		*tmp_list;
  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  tmp_list = gwf_project_get_ofrsig_list(project,name);
  return (tmp_list) ? (offersig*)(tmp_list->data) : NULL;
}

static GList*
gwf_project_get_ofrsig_list (GwfProject *project,gchar *name)
{
  GwfSignalData	*data;

  data = gwf_project_signal_get_struct(project);
  g_return_val_if_fail(data != NULL,NULL);

  return g_list_find_custom(data->list, (gpointer)name, (GCompareFunc)gwf_offersig_cmp);
}

static GwfSignalData*
gwf_project_signal_get_struct (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);

  return (GwfSignalData*)gwf_proj_data_get_data(gwf_data_list_get(gwf_project_get_data_list(project),"Signals"));
}

/**********************/
/* Signal marshallers */
/**********************/

static void
gwf_marshal_NONE_STRING_STRING	(GtkObject *object,GtkSignalFunc func,gpointer func_data,GtkArg	*args)
{
  GtkSignal_NONE__STRING_STRING rfunc;
  
  rfunc = (GtkSignal_NONE__STRING_STRING)func;
  (*rfunc)(object,GTK_VALUE_STRING(args[0]),GTK_VALUE_STRING(args[1]),func_data);
}

/************************/
/* GList free functions */
/************************/

static void
gwf_param_list_free (GList **list)
{
  GList		*tmp_list;
  
  for ( tmp_list = *list ; tmp_list ; tmp_list = g_list_next(tmp_list) )
  {
    g_free( ((sigparam*)tmp_list->data)->type );
    g_free( ((sigparam*)tmp_list->data)->name );
    g_free( tmp_list->data);
  }
  g_list_free(*list);
  *list = NULL;
}

static void
gwf_offersig_list_free (GList **list)
{
  GList		*tmp_list;

  for ( tmp_list = *list ; tmp_list ; tmp_list = g_list_next(tmp_list) )
  {
    g_free( ((offersig*)tmp_list->data)->name );
    g_free( ((offersig*)tmp_list->data)->retval );
    gwf_param_list_free(&(((offersig*)tmp_list->data)->param));
    g_free( tmp_list->data);
  }
  g_list_free(*list);
  *list = NULL;
}

/***************************/
/* GList compare functions */
/***************************/

static gint
gwf_offersig_cmp (gconstpointer a, gconstpointer b)
{
  return strcmp( ((offersig*)a)->name , (gchar*) b);
}

static gint
gwf_param_cmp (gconstpointer a,gconstpointer b)
{
  return strcmp( ((sigparam*)a)->name, (gchar*) b);
}

/*********************/
/* Foreach functions */
/*********************/

static void
foreach_ofrsig (gpointer data,gpointer user_data)
{
  GwfOfrsigForeach	*tmp = user_data;
  
  tmp->func((offersig*)data,tmp->user_data);
}

static void
foreach_sigparam (gpointer data,gpointer user_data)
{
  GwfSigparamForeach	*tmp = user_data;
  
  tmp->func((sigparam*)data,tmp->user_data);
}
