/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_xmlio_signal.h"
#include <gtk/gtk.h>
#include "gwf_projdata_signal.h"
#include <gwf_lib.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

enum
{
  GWF_OFRSIG,

  GWF_OFRSIG_NAME,
  GWF_OFRSIG_RETVAL,
  GWF_OFRSIG_RVTYPE,
  GWF_OFRSIG_PARAM,
  GWF_PARAM_NAME,
  GWF_PARAM_TYPE,
  GWF_PARAM_GTKTYPE,

  LAST_OFRSIG_TAG
};

static gint		gwf_xmlio_parse_ofrsig		(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error);
static void		gwf_xmlio_add_ofrsig_foreach	(offersig *sig,gpointer user_data);
static void		gwf_xmlio_add_param_foreach	(sigparam *param,gpointer user_data);

static gchar		*xml_tags[LAST_OFRSIG_TAG]=	{
							"OFFERSIG",
        						"NAME",
               						"RETVAL",
               						"RVTYPE",
							"OFRSIGPARAM",
                      					"NAME",
                            				"TYPE",
                            				"GTKTYPE"
							};


gint
gwf_xmlio_load_ofrsig(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error)
{
  gint		ok = 1;

  g_return_val_if_fail(cur != NULL,0);
  g_return_val_if_fail(xml_io != NULL,0);
  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(error != NULL,0);

  while (cur && ok)
  {
    if ((strcmp(cur->name,xml_tags[GWF_OFRSIG]) == 0) && (cur->ns == ns))
      ok = gwf_xmlio_parse_ofrsig(cur,ns,xml_io,project,error);
    else
    {
      gwf_error_raise_error(error,"XmlIO : Unknown tag",cur->name);
      return 0;
    }
    cur = cur->next;
  }

  return ok;
}
							
gint
gwf_xmlio_save_ofrsig(xmlNodePtr parent,xmlNsPtr ns,GwfProject *project)
{
  XmlSaveData		save_data;

  save_data.parent = parent;
  save_data.result = 1;
  save_data.ns = ns;
  save_data.project = project;

  gwf_project_ofrsig_foreach(project,gwf_xmlio_add_ofrsig_foreach,(gpointer)&save_data);
  return save_data.result;
}

static gint
gwf_xmlio_parse_ofrsig(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error)
{
  offersig	*signal;
  sigparam	param;
  gchar		*tmp;
  gint		result;

  g_return_val_if_fail(cur != NULL,0);
  g_return_val_if_fail(xml_io != NULL,0);
  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(error != NULL,0);

  signal = g_malloc(sizeof(offersig));

  signal->name = xmlGetProp(cur,xml_tags[GWF_OFRSIG_NAME]);
  if (!signal->name)
  {
    gwf_error_raise_error(error,"XmlIO : Signal no name");
    return 0;
  }

  signal->retval = xmlGetProp(cur,xml_tags[GWF_OFRSIG_RETVAL]);
  signal->rvtype = string_to_gtk_type(xmlGetProp(cur,xml_tags[GWF_OFRSIG_RVTYPE]));
  signal->param = NULL;

  cur = cur->childs;

  while (cur)
  {
    if ((strcmp(cur->name,xml_tags[GWF_OFRSIG_PARAM]) == 0) && (cur->ns == ns))
    {
      param.name = xmlGetProp(cur,xml_tags[GWF_PARAM_NAME]);
      param.type = xmlGetProp(cur,xml_tags[GWF_PARAM_TYPE]);
      tmp = xmlGetProp(cur,xml_tags[GWF_PARAM_GTKTYPE]);
      param.gtktype = string_to_gtk_type(tmp);
      g_free(tmp);

      if (!param.name)
        gwf_error_raise_error(error,"XmlIO : Param no name",signal->name);

      result = gwf_ofrsig_add_param(signal,&param);

      if (!result)
        gwf_error_raise_error(error,"XmlIO : Duplicate param name",signal->name);

      if (!param.name || !result)
      {
        g_free(param.type);
        gwf_ofrsig_free(signal);
        return 0;
      }
    }
    else
    {
      gwf_error_raise_error(error,"XmlIO : Unknown tag",cur->name);
      return 0;
    }

    cur = cur->next;
  }

  gwf_project_add_ofrsig (project,signal);
  gwf_ofrsig_free(signal);

  return 1;
}

static void
gwf_xmlio_add_ofrsig_foreach(offersig *sig,gpointer user_data)
{
  xmlNodePtr	sign;
  XmlSaveData	*parent_data = user_data;
  XmlSaveData	data;

  data.ns = parent_data->ns;
  data.project = parent_data->project;
  data.result = 1;

  sign = xmlNewChild(parent_data->parent,data.ns,xml_tags[GWF_OFRSIG],NULL);
  xmlSetProp(sign,xml_tags[GWF_OFRSIG_NAME],sig->name);
  xmlSetProp(sign,xml_tags[GWF_OFRSIG_RETVAL],sig->retval);
  xmlSetProp(sign,xml_tags[GWF_OFRSIG_RVTYPE],gtk_type_to_string(sig->rvtype));

  data.parent = sign;
  gwf_ofrsig_param_foreach(sig,gwf_xmlio_add_param_foreach,(gpointer)&data);

  if (!data.result)
    parent_data->result = 0;
}

static void
gwf_xmlio_add_param_foreach(sigparam *param,gpointer user_data)
{
  xmlNodePtr	prm;
  XmlSaveData	*data = user_data;

  prm = xmlNewChild(data->parent,data->ns,xml_tags[GWF_OFRSIG_PARAM],NULL);
  xmlSetProp(prm,xml_tags[GWF_PARAM_NAME],param->name);
  xmlSetProp(prm,xml_tags[GWF_PARAM_TYPE],param->type);
  xmlSetProp(prm,xml_tags[GWF_PARAM_GTKTYPE],gtk_type_to_string(param->gtktype));
}
